/*
**++
**  FACILITY:
**	CACHE_LOGGER
**
**  ABSTRACT:
**      Shareable log file routines :
**      OPEN_LOG_FILE(), CLOSE_LOG_FILE(), WRITE_LOG_FILE( MESSAGE ).
**
**  AUTHOR:
**      Bill Teahan, University of Waikato, Hamilton, New Zealand
**
**  COPYRIGHT:
**      Copyright  1991
**--
**/

#include jpidef
#include prvdef
#include descrip
#include "logfab.h"
#include "lograb.h"

typedef struct {
    short itm_w_buflen;
    short itm_w_code;
    char *itm_t_bufadr;
    char *itm_t_retlen;
} ITEM_TYPE;

globaldef{"logrms"} noshare struct FAB logfab;          /* log file fab */
globaldef{"logrms"} noshare struct RAB lograb;          /* log file rab */
globaldef{"logrms"} noshare char logbuf[2048];          /* log message buffer */

/*
 *  Global LOGGER routines
 *
 */

static int log_file_open = 0;	/* Global flag to indicate if file is open */

int open_log_file(filename)
   char *filename;
{
  int i;
  char *fnamep, *namep;
  char fname[128];

  if (!log_file_open) {
    log_file_open = 0;

    fnamep = fname;
    while (*fnamep++ = *filename++);
    fnamep--;
    logfab.fab$b_bid = FAB$C_BID;
    logfab.fab$b_bln = FAB$C_BLN;
    logfab.fab$b_fac = FAB$M_PUT;
    logfab.fab$b_rfm = FAB$C_VAR;
    logfab.fab$l_fop = FAB$M_CIF;
    logfab.fab$l_fna = fname;
    logfab.fab$b_fns = fnamep - fname;
    logfab.fab$b_rat = FAB$M_CR;
    logfab.fab$b_org = FAB$C_SEQ;
    logfab.fab$w_deq = 256;
    logfab.fab$b_shr = FAB$M_SHRDEL | FAB$M_SHRGET | FAB$M_SHRPUT | FAB$M_SHRUPD;
   
    sys$create(&logfab);
  }
  return(1);
}

int write_log_file(message)
  char *message;
{
  char *username;
  char *account;
  char *log;
  char *time;
  char asctim[25];
  int asctim_len;
  $DESCRIPTOR( asctim_dsc, asctim );

  static int write_init = 0;

  if (!write_init) {
    write_init = 1;

    /* Set up RAB for log file */
    lograb.rab$b_bid = RAB$C_BID;
    lograb.rab$b_bln = RAB$C_BLN;
    lograb.rab$l_fab = &logfab;
    lograb.rab$l_rop = RAB$M_EOF;		/* Append to file */
    lograb.rab$l_rbf = lograb.rab$l_ubf = &logbuf;
    lograb.rab$w_rsz = lograb.rab$w_usz = sizeof logbuf;
  
    if (!(sys$connect(&lograb) & 1)) {
      sys$close(&logfab);
      return(0);
    }
  }
  sys$asctim( &asctim_len, &asctim_dsc, 0, 0 );
  asctim[20] = '\0';
  log = logbuf;
  time = asctim;

  /* copy username, account name & message to record output buffer */
  while (*time)  *log++ = *time++;
  *log++ = ' ';
  while (*message)  *log++ = *message++;
  lograb.rab$w_rsz = log - logbuf;
  return( sys$put(&lograb));
}

int close_log_file()
{
  log_file_open = 0;
  return( sys$close(&logfab));
}
