/*
**++
**  FACILITY:
**      NNTP_TTY
**
**  ABSTRACT:
**      This module contains the QIO interface to the terminal for an
**      interactive interface via NNTP to the local database. (This module
**      is used to check the functionality of the NNTP_SERVER code.
**
**  AUTHOR:
**      Geoff Huston
**
**  COPYRIGHT:
**      Copyright  1988,1989,1990
**--
**/

#module NNTP_TTY "V6.0"

#include signal

#define TIMEOUT     300         /* read wait timeout  - 5 minutes */

int (*next_function)();

/*
 *  cancel_net
 *
 *  If the read timer expires then cancel this server task (by exiting)
 */

cancel_net()
{
  write_net("400 service discontinued - read timeout\r\n");
  log_to_file(1);
  exit(1);
}

/*
 *  read_net
 *
 *  read from the client process - if the read timer expires cancel this server
 */

read_net(buffer,size,unit)
  char *buffer;
  int size;
  int unit;
{
  signal(SIGALRM,cancel_net);
  alarm(TIMEOUT);
  if (!gets(buffer)) strcpy(buffer,"quit");
  alarm(0);
  strcat(buffer,"\n");
  return(strlen(buffer));
}

/*
 *  write_net
 *
 *  write to the client process - catch i/o errors and abort
 */

write_net(s,unit)
  char *s;
  int unit;
{
  printf("%s",s);
}

/*
 *  next_call
 *
 *  Set the next function to be invoked by the server
 */

next_call(unit,func,type)
  int unit, (*func)(), type;
{
  next_function = func;
}

/*
 *  open_net
 *
 *  Open tty channel
 */

open_net()
{
    return(1);
}

/*
 *  close_net
 *
 *  Close the tty link - noop for tty channel
 */

close_net()
{
  log_to_file(1);
}

/*
 *  getremhost
 *
 *  Return the remote host name and remote username
 */

getremhost(remhost,remuser,unit)
  char *remhost, *remuser;
  int unit;
{
  char *cp;

  if (remhost) {
    cp = news_getenv("SYS$NODE",1);
    if (cp) {
      strcpy(remhost,cp);
      if (cp = strchr(remhost,':')) *cp = '\0';
      }
    }
  if (remuser) cuserid(remuser);
}

/*               
 *  main
 */

main()
{
  if (!server_init(1)) exit(1);
  if (!open_net()) exit(1);
  server_init_unit(1);
  while (next_function) (*next_function)(1);
  server_shut();
  close_net();
}
